# DelayedArray [<img src="https://bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="200" align="right"/>](https://bioconductor.org/)

[![in_Bioc](https://bioconductor.org/shields/years-in-bioc/DelayedArray.svg)](https://bioconductor.org/packages/DelayedArray)
[![rank](https://bioconductor.org/shields/downloads/release/DelayedArray.svg)](https://bioconductor.org/packages/stats/bioc/DelayedArray)

**DelayedArray** is an R/Bioconductor package that provides a unified framework
for working transparently with on-disk and in-memory array-like datasets.

See <https://bioconductor.org/packages/DelayedArray> for more information,
including for how to install the release version of the package (please refrain
from installing directly from GitHub).

|BioC version|Status|
|------------|------|
| release | [![build](https://bioconductor.org/shields/build/release/bioc/DelayedArray.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/DelayedArray/) [![platforms](https://bioconductor.org/shields/availability/release/DelayedArray.svg)](https://bioconductor.org/packages/release/DelayedArray#archives) [![dependencies](https://bioconductor.org/shields/dependencies/release/DelayedArray.svg)](https://bioconductor.org/packages/release/DelayedArray#since) |
| devel   | [![build](https://bioconductor.org/shields/build/devel/bioc/DelayedArray.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/DelayedArray/) [![platforms](https://bioconductor.org/shields/availability/devel/DelayedArray.svg)](https://bioconductor.org/packages/devel/DelayedArray#archives) [![dependencies](https://bioconductor.org/shields/dependencies/devel/DelayedArray.svg)](https://bioconductor.org/packages/devel/DelayedArray#since) |

